# Function prototype：

HRESULT STDMETHODCALLTYPE GetSubFolder(long lSessionId, LPCWSTR szwParentId, FCI_FOLDER pGroupIds[],int nSize,int* pnCount,FCISTATUS* pStatus);

HRESULT STDMETHODCALLTYPE GetFolderTags(long lSessionId, LPCWSTR szwParentId, FCI_TAG pTags[],int nSize,int* pnCount,FCISTATUS* pStatus);

FCI_FOLDER 和 FCI_TAG Structure declared in DataDef. h；

When the client calls the GetSubFolder function, the szwParentId and pGroupIds arrays are provided; The client needs to actively release the memory of the string pointer of pGroupIds: CoTaskMemFree();

The GetFolderTags function is similar to GetSubFolder.


# Code example：

```plain
LPWSTR m_pszwParentID;

HRESULT FCINTFClient::GetSubFolder()
{
    deleteLPWSTR(m_pszwParentID);
    m_pszwParentID = (LPWSTR)enterStringConvertWChar(string("Enter Parent Folder ID:"));
#define FOLDER_COUNT_MAX 5
    FCI_FOLDER pGroupIds[FOLDER_COUNT_MAX];
    int size = 0;

    hResult = m_pCalibratorDownload->GetSubFolder(sessionId, m_pszwParentID, pGroupIds, FOLDER_COUNT_MAX, &size, &status);
    if (hResult != S_OK)
    {
        std::cout << "failed\n";
        return hResult;
    }
    std::cout << "SubFolder size: " << size << std::endl;
    for (int i = 0; i < size; i++)
    {
        char* ID = CHelper::LPWSTRToChar(pGroupIds[i].szwID);
        char* name = CHelper::LPWSTRToChar(pGroupIds[i].szwName);
        std::cout << name << ":" << ID << std::endl;
        delete[] ID;
        delete[] name;
    }

    std::cout << "status: " << status << std::endl;

    // free
	for (int i = 0; i < size; i++)
	{
		deleteLPWSTR(pGroupIds[i].szwID);
		deleteLPWSTR(pGroupIds[i].szwName);
		deleteLPWSTR(pGroupIds[i].szwParentID);
	}

    return hResult;
#undef FOLDER_COUNT_MAX
}
```
# Appendix：CalibratorInterface.h

```plain
#pragma once
#include <unknwn.h>
#include "DataDef.h"

#define DEVICE_ADT760		(0x01)
#define DEVICE_ADT761A		(0x02)
#define DEVICE_ADT227		(0x03)

#define DEVICE_MODEL		DEVICE_ADT227

#if DEVICE_MODEL == DEVICE_ADT760
// E8389AED-7B15-11D1-B326-00001CBE02AA
_declspec(selectany) GUID CLSID_CalibratorInfo =
{ 0xe8389aed,0x7b15,0x11d1,{0xb3,0x26,0x0,0x0,0x1c,0xbe,0x02,0xaa} };//生成不同设备对应的COM组件时，修改这个GUID(会在注册表中生成对应的注册表项，所以不能重复)
//Device Type
_declspec(selectany) char* g_deviceFlag = "ADT760";

#elif DEVICE_MODEL == DEVICE_ADT761A
// E8390AED-7B15-11D1-B326-00001CBE02AA
_declspec(selectany) GUID CLSID_CalibratorInfo =
{ 0xe8390aed,0x7b15,0x11d1,{0xb3,0x26,0x0,0x0,0x1c,0xbe,0x02,0xaa} };//生成不同设备对应的COM组件时，修改这个GUID(会在注册表中生成对应的注册表项，所以不能重复)
//Device Type
_declspec(selectany) char* g_deviceFlag = "ADT761A";

#elif DEVICE_MODEL == DEVICE_ADT227
// E8391AED-7B15-11D1-B326-00001CBE02AA
_declspec(selectany) GUID CLSID_CalibratorInfo =
{ 0xe8391aed,0x7b15,0x11d1,{0xb3,0x26,0x0,0x0,0x1c,0xbe,0x02,0xaa} };//生成不同设备对应的COM组件时，修改这个GUID(会在注册表中生成对应的注册表项，所以不能重复)
//Device Type
_declspec(selectany) char* g_deviceFlag = "ADT227";

#endif



//FF5EFD41-7B15-11D1-B326-00001CBE02AA
_declspec(selectany) GUID IID_CalibratorInfo =
{ 0xff5efd41,0x7b15,0x11d1,{0xb3,0x26,0x0,0x0,0x1c,0xbe,0x02,0xaa} };

// FF5EFD42-7B15-11D1-B326-00001CBE02AA
_declspec(selectany) GUID IID_CalibratorDownload =
{ 0xff5efd42,0x7b15,0x11d1,{0xb3,0x26,0x0,0x0,0x1c,0xbe,0x02,0xaa} };

// FF5EFD43-7B15-11D1-B326-00001CBE02AA
_declspec(selectany) GUID IID_CalibratorUpload =
{ 0xff5efd43,0x7b15,0x11d1,{0xb3,0x26,0x0,0x0,0x1c,0xbe,0x02,0xaa} };

class ICalibratorInfo : public IUnknown
{
public:
	virtual HRESULT STDMETHODCALLTYPE DriverProperties(FCISTATUS* pStatus) = 0;//必须为虚函数
	virtual HRESULT STDMETHODCALLTYPE Open(int nPortNumber, long* plSessionId, long* plCapabilities, FCISTATUS* pStatus) = 0;
	virtual HRESULT STDMETHODCALLTYPE Close(long lSessionId, FCISTATUS* pStatus) = 0;
	virtual HRESULT STDMETHODCALLTYPE Properties(long lSessionId, FCISTATUS* pStatus) = 0;
	virtual HRESULT STDMETHODCALLTYPE GetId(long lSessionId, LPWSTR* pszwCalManufacturer, LPWSTR* pszwCalModel, LPWSTR* pszwCalSerialNum, FCISTATUS* pStatus) = 0;
	virtual HRESULT STDMETHODCALLTYPE GetCalDates(long lSessionId, SYSTEMTIME* pLastCalDate, SYSTEMTIME* pNextCalDueDate, FCISTATUS* pStatus) = 0;
	virtual HRESULT STDMETHODCALLTYPE SetDateAndTime(long lSessionId, FCISTATUS* pStatus) = 0;
	virtual HRESULT STDMETHODCALLTYPE SetTempStandard(long lSessionId, int nTemperatureStd, FCISTATUS* pStatus) = 0;
	virtual HRESULT STDMETHODCALLTYPE GetTestResultsCount(long lSessionId, int* pnCount, FCISTATUS* pStatus) = 0;
	virtual HRESULT STDMETHODCALLTYPE ClearMemory(long lSessionId, FCISTATUS* pStatus) = 0;
	virtual HRESULT STDMETHODCALLTYPE ValidateTag(FCICOMMDEF* pCommDef, FCIDEVIN* pInputDef, FCIDEVOUT* pOutputDef, FCIDEVRELATION* pRelationDef, FCITEST* pTestDef, FCISTATUS* pStatus) = 0;
};

class ICalibratorDownload : public IUnknown
{
public:
	virtual HRESULT STDMETHODCALLTYPE StartDownloading(long lSessionId, LPCWSTR szSessionName, FCISTATUS* pStatus) = 0;//必须为虚函数
	virtual HRESULT STDMETHODCALLTYPE DownloadTag(long lSessionId, LPCWSTR szwDeviceTag, LPCWSTR szwDeviceSerialNum, FCICOMMDEF* pCommDef, FCIDEVIN* pInputDef, FCIDEVOUT* pOutputDef, FCIDEVRELATION* pRelationDef, FCITEST* pTestDef, LPCWSTR szwSetupInstructions, LPCWSTR szwCleanupInstructions, LPWSTR* pszwLocationInCalibrator, FCISTATUS* pStatus) = 0;
	virtual HRESULT STDMETHODCALLTYPE FinishDownloading(long lSessionId, FCISTATUS* pStatus) = 0;
	virtual HRESULT STDMETHODCALLTYPE AbortDownload(long lSessionId, FCISTATUS* pStatus) = 0;
	virtual HRESULT STDMETHODCALLTYPE DeleteTag(long lSessionId, LPCWSTR szwLocationInCalibrator, FCISTATUS* pStatus) = 0;
	virtual HRESULT STDMETHODCALLTYPE AddFolder(long lSessionId, LPCWSTR szwFolderName, LPCWSTR szwParentId, LPWSTR* pszwGroupId,FCISTATUS* pStatus) = 0;
	virtual HRESULT STDMETHODCALLTYPE DeleteFolder(long lSessionId, LPCWSTR szwGroupId, FCISTATUS* pStatus) = 0;
	virtual HRESULT STDMETHODCALLTYPE GetFolder(long lSessionId, LPCWSTR szwFolderName, LPCWSTR szwParentId, LPWSTR* pszwGroupId,FCISTATUS* pStatus) = 0;
	virtual HRESULT STDMETHODCALLTYPE EnterFolder(long lSessionId, LPCWSTR szwGroupId, FCISTATUS* pStatus) = 0;
	virtual HRESULT STDMETHODCALLTYPE GetSubFolder(long lSessionId, LPCWSTR szwParentId, FCI_FOLDER pGroupIds[],int nSize,int* pnCount,FCISTATUS* pStatus) = 0;
	virtual HRESULT STDMETHODCALLTYPE GetFolderTags(long lSessionId, LPCWSTR szwParentId, FCI_TAG pTags[],int nSize,int* pnCount,FCISTATUS* pStatus) = 0;
};

class ICalibratorUpload : public IUnknown
{
public:
	virtual HRESULT STDMETHODCALLTYPE StartUploading(long lSessionId, FCISTATUS* pStatus) = 0;
	virtual HRESULT STDMETHODCALLTYPE UploadNextTag(long lSessionId, LPWSTR* pszwLocationInCalibrator, LPWSTR* pszwDeviceTag, LPWSTR* pszwDeviceSerialNum, LPWSTR* pszwTechnician, LPWSTR* pszwServiceNote, int* pnTemperatureStd, FCICOMMDEF* pCommDef, FCIRESULT* pAsFound, FCIRESULT* pAsLeft, FCISTATUS* pStatus) = 0;
	virtual HRESULT STDMETHODCALLTYPE FinishUploading(long lSessionId, FCISTATUS* pStatus) = 0;
	virtual HRESULT STDMETHODCALLTYPE AbortUpload(long lSessionId, FCISTATUS* pStatus) = 0;
	virtual HRESULT STDMETHODCALLTYPE UploadNextResult(long lSessionId, LPWSTR* pszwLocationInCalibrator, LPWSTR* pszwDeviceTag, LPWSTR* pszwDeviceSerialNum, LPWSTR* pszwTechnician, LPWSTR* pszwServiceNote, int* pnTemperatureStd, FCICOMMDEF* pCommDef, FCIRESULT* pAsFound, FCIRESULT* pAsLeft, FCISTATUS* pStatus) = 0;
};
```
# Appendix：DataDef.h

```plain
#pragma once

/***************************************************************************
* GUID and strings for the "FieldCalibrators" component category.
***************************************************************************/
DEFINE_GUID(CATID_FieldCalibrators, 0xff5efd40, 0x7b15, 0x11d1, 0xb3, 0x26, 0x00, 0x00, 0x1c, 0xbe, 0x02, 0xaa);
#define CATIDFIELDCAL {0xff5efd40, 0x7b15, 0x11d1, 0xb3, 0x26, 0x00, 0x00, 0x1c, 0xbe, 0x02, 0xaa}
#define OLEFCICATEGORY L"Field Calibrators"

/***************************************************************************
* Status Codes
* All calibrator functions return a standard HRESULT that indicates whether
* the function call succeeded from a COM standpoint.
* Function specific status is placed in the pStatus variable associated with
* each function. The possible values are enumerated here.
***************************************************************************/
typedef enum tagFCISTATUS
{
	FCI_OK = 0, // No error.
	FCI_FAILED = 1, // The function failed for an unspecified reason.
	FCI_COMM_ERROR = 2, // Unable to communicate with the calibrator.
	FCI_REVISION_ERROR = 3, // The calibrator driver revision is incompatible
	// with this calibrator.
	FCI_CANCELED = 4, // The operation was canceled.
	FCI_NOT_SUPPORTED = 5, // This calibrator does not support this function.
	FCI_ALREADY_OPEN = 6, // There is already a calibrator open.
	FCI_WRONG_SESSION = 7, // The session identifier is incorrect.
	FCI_WRONG_CALIBRATOR = 8, // The calibrator currently attached is not the one associated
	// with this session.
	FCI_ERR_IN_RANGE = 9, // The input range is not supported by this calibrator.
	FCI_ERR_IN_UNITS = 10, // The input engineering unit is not supported by this calibrator.
	FCI_ERR_IN_TYPE = 11, // The input type is not supported by this calibrator.
	FCI_ERR_IN_MANUAL = 12, // This calibrator does not support manual entry for input.
	FCI_ERR_OUT_RANGE = 13, // The output range is not supported by this calibrator.
	FCI_ERR_OUT_UNITS = 14, // The output engineering unit is not supported by this calibrator.
	FCI_ERR_OUT_TYPE = 15, // The input type is not supported by this calibrator.
	FCI_ERR_OUT_MANUAL = 16, // This calibrator does not support manual entry for output.
	FCI_ERR_INOUT = 17, // Invalid combination of input and output types for this calibrator.
	FCI_ERR_RELATION = 18, // The input to output relation is not supported by this calibrator.
	FCI_ERR_PROBE = 19, // The temperature probe is not supported by this calibrator.
	FCI_ERR_CJC = 20, // The cold junction compensation type is not supported.
	FCI_ERR_ENUM = 21, // A given enumeration is not a valid value.
	FCI_ERR_RESOURCE = 22, // Driver was unable to load a required resource.
	FCI_ERR_TESTPOINT = 23, // Error in one or more test points.
	// Out of range, not stable, or not valid.
	FCI_ERR_TESTTYPE = 24, // The test type does not match the instrument or is not supported.
	FCI_ERR_POWER = 25, // The loop power configuration is inconsistent or not supported.
	FCI_ERR_TAG = 26, // A tag is required for this operation.
	FCI_MEM_FULL = 27, // The calibrator memory is full.
	FCI_MEM_EMPTY = 28,
	FCI_MEM_HAS_DATA = 29, // The calibrator memory contains test data.
	FCI_END_OF_UPLOAD = 30, // End of upload list has been reached.
	FCI_ERR_DOWNLOAD = 31, // Error encountered when trying to finalize download.
	FCI_ERR_TAGLENGTH = 32, // The tag length sent exceeds the length supported by this driver.
	FCI_ERR_SNLENGTH = 33, // The serial number is longer than the max supported by this driver.
	FCI_ERR_MAXPOINTS = 34, // There are more AF or AL points than can be uploaded in a packet.
	FCI_END
} FCISTATUS;

/***************************************************************************
* Calibrator Capabilities
* These are bitfield definitions returned in the lCapabilities parameter.
* If a bit is set, then the calibrator can support the associated activity.
***************************************************************************/
#define FCI_CAP_ID 0x01L // Read calibrator manufact, model, & serial
#define FCI_CAP_CALINFO 0x02L // Read calibrator last cal and next cal due
#define FCI_CAP_DATESET 0x04L // Set the calibrator date & time
#define FCI_CAP_TEMPSTD 0x08L // Set the calibrator temperature standard
#define FCI_CAP_MEMCLEAR 0x10L // Clear calibrator memory
#define FCI_CAP_DOWNLOAD 0x20L // Download test setups to the calibrator
#define FCI_CAP_DLSETUP 0x40L // Send text prompt to show before test
#define FCI_CAP_DLCLEANUP 0x80L // Send text prompt to show after test
#define FCI_CAP_TECHNAME 0x100L // Supplies technician name with results
#define FCI_CAP_UPLOAD 0x200L // Upload test data from the calibrator

/***************************************************************************
* Definitions for various discreet values.
***************************************************************************/
#define FCI_MAX_TEST_POINTS 21
// Temperature standard indicators
#define STD_IPTS68 1
#define STD_ITS90 2
// Contact definitions for switches
#define CONTACT_TYPE_WET 0
#define CONTACT_TYPE_DRY 1
#define CONTACT_FORM_NO 0
#define CONTACT_FORM_NC 1
#define CONTACT_FORM_C 2
#define CONTACT_TRIP_INC 0
#define CONTACT_TRIP_DEC 1
//Manual entry vs calibrator readings for testing
#define BY_USER 1
#define BY_CALIBRATOR 2
//Power for device under test
#define PWR_EXTERNAL 1
#define PWR_CALIBRATOR 2
#define PWR_CALIBRATOR_HIV 3
//Float value to represent invalid floating point number.
//This number is equivalent to FLT_MAX
#define INVALID_FLOAT 3.402823466e+38F
////////////////////////////////////////////////////////////////////////////

/***************************************************************************
* This first major section provides a method of defining an instrument for
* a calibrator.
* Each instrument definition is composed of the following blocks.
* Communication - defines any comm capabilities
* Input - defines the sensor input to the instrument
* Output - defines the output generated by the instrument
* Relation - defines the expected relation between input and output.
***************************************************************************/
////////////////////////////////////////////////////////////////////////////

/***************************************************************************
* Communication Block Definitions
***************************************************************************/
/*
* Communication block types
*/
typedef enum tagFCICOMMTYPE
{
	FCI_COMMTYPE_NONE = 1, // No digital communication
	FCI_COMMTYPE_HART = 2 // Uses HART protocol
} FCICOMMTYPE;

/*
* Communication block structures
*/
typedef struct tagFCI_COMMDEF_HART
{
	BYTE cBlockId; // The transmitter variable associated with this block.
	// ASCII P, S, T, Q. Or 0 - 79 Decimal if not mapped to
	// analog output.
	BYTE cURev; // HART Universal Revision
	BYTE cPollAddr; // Poll Address
	// The following arrays are in HART message format,
	// they are not NULL terminated strings.
	BYTE acUid[5]; // Five Byte Unique Identifier. Set to 0 for URev = 4
	// This is the mfid, devid, and uid from HART cmd 0 or 11
	// The next three are from command 13,
	BYTE acTag[6]; // The packed ASCII tag field in the instrument.
	BYTE acDescriptor[12]; // The packed ASCII descriptor field
	BYTE acDate[3]; // The date field
	// From command 12,
	BYTE acMessage[24]; // The packed ASCII message field
} FCI_COMMDEF_HART;

typedef struct tagFCICOMMDEF
{
	FCICOMMTYPE CommType;
	union
	{
		FCI_COMMDEF_HART HartData;
		/* Empty union arm */
	};
} 	FCICOMMDEF;

/***************************************************************************
* Instrument Input/Output block types defined
* Instrument Input = sourced or generated by the calibrator.
* Instrument Output = measured by the calibrator.
***************************************************************************/
/* I/O Block Types */
typedef enum tagFCIBLOCKTYPE
{ // Blocks may be used for (relative to device under test)
	FCI_BLKTYPE_GENERIC = 1, // input or output
	FCI_BLKTYPE_TEMP_RTD = 2, // input: calibrator simulates an RTD
	FCI_BLKTYPE_TEMP_TC = 3, // input: calibrator simulates a thermocouple (TC)
	FCI_BLKTYPE_FREQUENCY = 4, // input or output
	FCI_BLKTYPE_PRESSURE = 5, // input or output
	FCI_BLKTYPE_TEMP_MEASRTD = 6, // input or output: use calibrator to measure temperature
	FCI_BLKTYPE_TEMP_MEASTC = 7, // input or output: use calibrator to measure temperature
	FCI_BLKTYPE_HART = 8, // input or output: set or measured via HART commands
	FCI_BLKTYPE_SWITCH = 9, // output
	FCI_BLKTYPE_DIFFTEMP = 10 // differential temperature
} FCIBLOCKTYPE;
typedef struct tagFCI_BLKDEF_PRESSURE
{
	FLOAT rURV; // Upper range value
	FLOAT rLRV; // Lower range value
	FLOAT rSettling; // Settling time (how many seconds to wait for stable reading)
	WORD wUnits; // Unit Code. See HART table 2.
	WORD wPressureType; // Type of pressure measurement. See HART table 19.
}FCI_BLKDEF_PRESSURE;
typedef struct tagFCI_BLKDEF_RTD
{
	FLOAT rURV; // Upper range value
	FLOAT rLRV; // Lower range value
	FLOAT rSettling; // Settling time (how many seconds to wait for stable reading)
	WORD wUnits; // Unit Code. See HART table 2.
	WORD wProbeType; // Temperature probe type. See HART table 16.
	WORD wNumWires; // Number of wires used by the RTD. Legal values are 2, 3, or 4.
	WORD wRESERVED;
}FCI_BLKDEF_RTD;

typedef struct tagFCI_BLKDEF_TC
{
	FLOAT rURV; // Upper range value
	FLOAT rLRV; // Lower range value
	FLOAT rSettling; // Settling time (how many seconds to wait for stable reading)
	FLOAT rManualCJC; // Value for manual cold junction compensation (NAN if not used)
	// Must be in same units as rURV and rLRV.
	WORD wUnits; // Unit Code. See HART table 2.
	WORD wProbeType; // Temperature probe type. See HART table 16.
	WORD wCJC; // Cold Junction Compensation Type. See HART table 18.
	WORD wProbeConnect; // How probe is connected. See HART table 17.
}FCI_BLKDEF_TC;

typedef struct tagFCI_BLKDEF_FREQUENCY
{
	FLOAT rURV; // Upper range value
	FLOAT rLRV; // Lower range value
	FLOAT rSettling; // Settling time (how many seconds to wait for stable reading)
	FLOAT rAmplitude; // Expected amplitude in volts (peak to peak)
	WORD wUnits; // Unit Code. See HART table 2.
	WORD wWaveForm; // Expected waveform. See HART table 20.
}FCI_BLKDEF_FREQUENCY;
typedef struct tagFCI_BLKDEF_GENERIC
{
	FLOAT rURV; // Upper range value
	FLOAT rLRV; // Lower range value
	FLOAT rSettling; // Settling time (how many seconds to wait for stable reading)
	WORD wUnits; // Unit Code. See HART table 2.
}FCI_BLKDEF_GENERIC;
typedef struct tagFCI_BLKDEF_SWITCH
{
	WORD wContactType; // 0=Wet(voltage present), 1=Dry(contacts only)
	WORD wForm; // 0=Normally Open(NO), 1=Normally Closed(NC), 2=Form C
	WORD wTripDirection; // 0=Trip on Increase, 1=Trip on Decrease
	FLOAT rWetContactVoltage; // expected contact voltage in Volts for wet contact type.
}FCI_BLKDEF_SWITCH;
/* useful only if calibrator has HART ability */
typedef struct tagFCI_BLKDEF_HART
{
	FLOAT rUSL; // Upper sensor limit
	FLOAT rLSL; // Lower sensor limit
	FLOAT rSettling; // Settling time (how many seconds to wait for stable reading)
	WORD wUnits; // Unit code. See HART table 2.
}FCI_BLKDEF_HART;
/* useful only if calibrator can communicate with the instrument */
typedef struct tagFCI_BLKDEF_DIFFTEMP //Differential Temperature
{
	FLOAT rURV; // Upper range value
	FLOAT rLRV; // Lower range value
	FLOAT rSettling; // Settling time (how many seconds to wait for stable reading)
	WORD wUnits; // Unit Code. See HART table 2.
	WORD wDeviceVariable1; // Diff Temp = DeviceVariable1 - DeviceVariable2
	WORD wDeviceVariable2; //
}FCI_BLKDEF_DIFFTEMP;

/***************************************************************************
* Instrument Input Block Definition (Calibrator Generated, or Sourced)
***************************************************************************/
typedef struct tagFCIDEVIN
{
	FCIBLOCKTYPE DevType;
	union
	{
		FCI_BLKDEF_PRESSURE PressureData;
		FCI_BLKDEF_RTD TempRtdData;
		FCI_BLKDEF_TC TempTcData;
		FCI_BLKDEF_RTD TempMeasRtdData;
		FCI_BLKDEF_TC TempMeasTcData;
		FCI_BLKDEF_FREQUENCY FrequencyData;
		FCI_BLKDEF_HART HartData;
		FCI_BLKDEF_DIFFTEMP DiffTempData;
		FCI_BLKDEF_GENERIC GenericData;
	};
} 	FCIDEVIN;

/***************************************************************************
* Instrument Output Block Definition (Calibrator Measurement)
***************************************************************************/
typedef struct tagFCIDEVOUT
{
	FCIBLOCKTYPE DevType;
	union
	{
		FCI_BLKDEF_PRESSURE PressureData;
		FCI_BLKDEF_RTD TempMeasRtdData;
		FCI_BLKDEF_TC TempMeasTcData;
		FCI_BLKDEF_FREQUENCY FrequencyData;
		FCI_BLKDEF_HART HartData;
		FCI_BLKDEF_SWITCH Switch;
		FCI_BLKDEF_GENERIC GenericData;
	};
} 	FCIDEVOUT;
/***************************************************************************
* Instrument Relation Block Definition
***************************************************************************/
/* These are a subset of HART Table 3 */
typedef enum tagFCIRELATIONTYPE
{
	FCI_RELTYPE_LINEAR = 0,
	FCI_RELTYPE_SQRT = 1,
	FCI_RELTYPE_TABLE = 4, // break point table
	FCI_RELTYPE_SWITCH = 230
} FCIRELATIONTYPE;

typedef struct tagFCI_RELDEF_SQRT
{
	FLOAT rBreakPoint; // Transition point from sqrt to linear, %input.
} FCI_RELDEF_SQRT;

typedef struct tagFCI_RELDEF_TABLE
{
	WORD wNumPoints; // Number of valid points in the table. Max of 30.
	WORD wInterpolate; // 1 = linear, 2 = spline
	FLOAT rInput[30]; // Array of input points. Pct of input span.
	FLOAT rOutput[30]; // Array of corresponding output points. Pct output span.
} FCI_RELDEF_TABLE;

typedef struct tagFCIDEVRELATION
{
	FCIRELATIONTYPE RelType;
	union
	{
		FCI_RELDEF_SQRT SqrtData;
		FCI_RELDEF_TABLE TableData;
		/* Empty union arm */
	};
} 	FCIDEVRELATION;

////////////////////////////////////////////////////////////////////////////
/***************************************************************************
* This second major section provides a means of sending a test
* procedure for an instrument to a calibrator, and for reading the test
* results from the calibrator.
***************************************************************************/
////////////////////////////////////////////////////////////////////////////
/***************************************************************************
* Test Definitions for proportional and switch instruments
***************************************************************************/
typedef enum tagFCITESTTTYPE
{
	FCI_TYPE_PROPORTIONAL = 1,
	FCI_TYPE_SWITCH = 2
} FCITESTTYPE;

typedef struct tagFCI_TEST_PROPORTIONAL
{
	WORD wReserved; // DUT = Device Under Test, i.e. the Instrument.
	WORD wPowerSource; // For DUT. PWR_EXTERNAL, PWR_CALIBRATOR, PWR_CALIBRATOR_HIV
	WORD wCalSourceInput; // Input to DUT generated: BY_USER, BY_CALIBRATOR
	WORD wCalReadInput; // Input to DUT measured: BY_USER, BY_CALIBRATOR
	WORD wCalMeasureOutput; // Output from DUT measured: BY_USER, BY_CALIBRATOR
	WORD wNumTestPoints; // Number of points to use when testing
	FLOAT arTestPoint[21]; // All point values are percent of input span
	FLOAT rTestPointTolerance; // % of span allowed off of test point
	FLOAT rAdjustmentLimit; // If adjusted, errors must fall below this limit
	FLOAT rMaxErrorLimit; // Max allowed error (fail if >) in percent of range
} FCI_TEST_PROPORTIONAL;

/*
* Unless otherwise noted, all of the values in this structure are expressed
* in the Engineering Units specified by the input block.
* It is expected that the calibrator will start at the input block rLRV
* and ramp its output toward the rURV over the rRampTime indicated.
* After a trip is detected, if bTestReset is true, then the calibrator should
* reverse the direction of the sensor input and record the reset point.
*/
typedef struct tagFCI_TEST_SWITCH
{
	FLOAT rTripSetPoint; // Expected trip point in Eng. units of the input.
	FLOAT rTripTolerance; // +/- allowed deviation in Eng units from trip
	FLOAT rResetDeadband; // expected difference between the trip and reset points.
	FLOAT rDeadbandTolerance; // +/- deviation allowed in the deadband value
	FLOAT rRampTime; // in seconds.
	BOOL bTestReset; // set to TRUE to test both Trip and Reset points.
} FCI_TEST_SWITCH;
typedef struct tagFCITEST
{
	FCITESTTYPE TestType;
	union
	{
		FCI_TEST_PROPORTIONAL Test;
		FCI_TEST_SWITCH SwitchTest;
		/* Empty union arm */
	};
} 	FCITEST;
/***************************************************************************
* Test Results for proportional and switch instruments.
***************************************************************************/
typedef struct tagFCI_RESULT_PROPORTIONAL
{
	double dInputLowerRangeValue; // Input and output are from the DUT perspective.
	double dInputUpperRangeValue; // Input is TO the instrument FROM Calibrator.
	int nInputRangeUnits;
	double dOutputLowerRangeValue; // Output is FROM the instrument TO Calibrator.
	double dOutputUpperRangeValue;
	int nOutputRangeUnits;
	int nRelationship; // Transfer Function - relation of input to output
	int nNumberOfTestPoints; // Actual number of test points used
	double adInput[21]; // Test Point Input (Engineering Units)
	double adOutput[21]; // Associated Output (Engineering Units)
	LPWSTR szwAuxEquipManufacturer; // Auxilliary equipment used in test.
	LPWSTR szwAuxEquipModel; // May be null if not used.
	LPWSTR szwAuxEquipSerialNum;
	double dAmbientTemperature; // Set to INVALID_FLOAT if calibrator can't supply.
	int nAmbientTemperatureUnits; // Set to unknown if calibrator can't supply.
	SYSTEMTIME TestDate; // Date and time of test
} FCI_RESULT_PROPORTIONAL;

/*
* In the switch result structure, use INVALID_FLOAT in either rTrip or rResetPoint to indicate
* that the point failed (did not change state within the test parameters).
*/
typedef struct tagFCI_RESULT_SWITCH
{
	WORD wUnits; // Unit code associated with the switch
	float rTripPoint; // Engineering unit value where switch transitions.
	float rResetPoint; // Engineering unit value where switch resets.
	WORD bResetTested; // Set to true if the reset point was tested.
	SYSTEMTIME TestDate; // Date and time of test
}FCI_RESULT_SWITCH;
typedef struct tagFCIRESULT
{
	FCITESTTYPE ResultType;
	union
	{
		FCI_RESULT_PROPORTIONAL Results;
		FCI_RESULT_SWITCH SwitchResults;
		/* Empty union arm */
	};
} 	FCIRESULT;

typedef struct tagFCI_FOLDER
{
	LPWSTR szwID;
	LPWSTR szwName;
	LPWSTR szwParentID;
} FCI_FOLDER;

typedef struct tagFCI_TAG
{
	LPWSTR szwParentID;
	LPWSTR szwLocationInCalibrator;
	LPWSTR szwDeviceTag;
	LPWSTR szwDeviceSerialNum;
} FCI_TAG;

```


